@@ -1,6 +1,6 @@
 package Bot::BasicBot::Pluggable::Module::Auth;
 BEGIN {
-  $Bot::BasicBot::Pluggable::Module::Auth::VERSION = '0.91';
+  $Bot::BasicBot::Pluggable::Module::Auth::VERSION = '0.92';
 }
 use base qw(Bot::BasicBot::Pluggable::Module);
 use warnings;
@@ -15,11 +15,21 @@ sub init {
             allow_anonymous => 0,
         }
     );
+    # A list of admin commands handled by this module and their usage
+    $self->{_admin_commands} = {
+        auth     => '<username> <password>',
+        adduser  => '<username> <password>',
+        deluser  => '<username>',
+        password => '<old password> <new password>',
+        users    => '',
+    };
 }
 
 sub help {
-    return
-"Authenticator for admin-level commands. Usage: !auth <username> <password>, !adduser <username> <password>, !deluser <username>, !password <old password> <new password>, !users.";
+    my $self = shift;
+    return "Authenticator for admin-level commands. Usage: "
+        . join ", ", map { "!$_ $self->{_admin_commands}{$_}" }
+            keys %{ $self->{_admin_commands} };
 }
 
 sub admin {
@@ -31,6 +41,28 @@ sub admin {
     # we don't care about commands that don't start with '!'.
     return 0 unless $body =~ /^!/;
 
+    # Find out what the command is:
+    my ($command, $params) = split '\s+', $mess->{body}, 2;
+    $command =~  s/^!//;
+    $command = lc $command;
+    my @params;
+    @params = split /\s+/, $params if defined $params;
+
+    # If it's not a command we handle, go no further:
+    return 0 unless exists $self->{_admin_commands}{$command};
+
+    # Basic usage check: the usage message declares which params are taken, so
+    # check we have the right number:
+    my $usage_message = $self->{_admin_commands}{$command};
+    
+    # Count how many params we want (assignment to empty list gets us list
+    # context, then assigning to scalar results in the count):
+    my $want_params = () =  $usage_message =~ m{<.+?>}g;
+
+    if (scalar @params != $want_params) {
+        return "Usage: !$command $usage_message";
+    }
+
     # system commands have to be directly addressed...
     return 1 unless $mess->{address};
 
@@ -38,8 +70,8 @@ sub admin {
     return "Admin commands in privmsg only, please."
       unless !defined $mess->{channel} || $mess->{channel} eq 'msg';
 
-    if ( $body =~ /^!auth\s+(\w+)\s+(\w+)/ ) {
-        my ( $user, $pass ) = ( $1, $2 );
+    if ($command eq 'auth') {
+        my ( $user, $pass ) = @params;
         my $stored = $self->get( "password_" . $user );
 
         if ( _check_password($pass, $stored) ) {
@@ -55,13 +87,8 @@ sub admin {
             delete $self->{auth}{ $mess->{who} };
             return "Wrong password.";
         }
-    }
-    elsif ( $body =~ /^!auth/ ) {
-        return "Usage: !auth <username> <password>.";
-
-    }
-    elsif ( $body =~ /^!adduser\s+(\w+)\s+(\w+)/ ) {
-        my ( $user, $pass ) = ( $1, $2 );
+    } elsif ( $command eq 'adduser' ) {
+        my ( $user, $pass ) = @params;
         if ( $self->authed( $mess->{who} ) ) {
             $self->set( "password_" . $user, _hash_password($pass) );
             return "Added user $user.";
@@ -69,13 +96,8 @@ sub admin {
         else {
             return "You need to authenticate.";
         }
-    }
-    elsif ( $body =~ /^!adduser/ ) {
-        return "Usage: !adduser <username> <password>";
-
-    }
-    elsif ( $body =~ /^!deluser\s+(\w+)/ ) {
-        my $user = $1;
+    } elsif ( $command eq 'deluser' ) {
+        my ($user) = @params;
         if ( $self->authed( $mess->{who} ) ) {
             $self->unset( "password_" . $user );
             return "Deleted user $user.";
@@ -83,13 +105,8 @@ sub admin {
         else {
             return "You need to authenticate.";
         }
-    }
-    elsif ( $body =~ /^!deluser/ ) {
-        return "Usage: !deluser <username>";
-
-    }
-    elsif ( $body =~ /^!passw?o?r?d?\s+(\w+)\s+(\w+)/ ) {
-        my ( $old_pass, $pass ) = ( $1, $2 );
+    } elsif ( $command eq 'password' ) {
+        my ( $old_pass, $pass ) = @params;
         if ( $self->authed( $mess->{who} ) ) {
             my $username = $self->{auth}{ $mess->{who} }{username};
             if (_check_password($old_pass, $self->get("password_$username")) ) {
@@ -103,23 +120,15 @@ sub admin {
         else {
             return "You need to authenticate.";
         }
-    }
-    elsif ( $body =~ /^!passw?o?r?d?/ ) {
-        return "Usage: !password <old password> <new password>.";
-
-    }
-    elsif ( $body =~ /^!users/ ) {
+    } elsif ( $command eq 'users' ) {
         return "Users: "
           . join( ", ",
             map { my $user = $_; $user =~ s/^password_// ? $user : () }
               $self->store_keys( res => ["^password"] ) )
           . ".";
+    
     }
-    else {
-        return
-          if $self->get("allow_anonymous") || $self->authed( $mess->{who} );
-        return "You need to authenticate.";
-    }
+    
 }
 
 sub authed {
@@ -135,6 +144,7 @@ sub authed {
 # use salted hashed passwords.
 sub _check_password {
     my ($entered_pw, $stored_pw) = @_;
+    return unless defined $entered_pw && defined $stored_pw;
     if ($stored_pw =~ /^\{SSHA\}/) {
         return Crypt::SaltedHash->validate($stored_pw, $entered_pw);
     } else {
@@ -159,7 +169,7 @@ Bot::BasicBot::Pluggable::Module::Auth - authentication for Bot::BasicBot::Plugg
 
 =head1 VERSION
 
-version 0.91
+version 0.92
 
 =head1 SYNOPSIS
 
